####################################################################################
##################  Two-block partial least-squares analysis		################
##################  following Rohlf and Corti 2000 Syst. Biol.		################
##################	49:740-753
####################################################################################

############	INPUT	###############
### (1) "data" is a dataframe with only continuous data (i.e., that which is to be analyzed and nothing more)
### (2) "nvar1" is the number of variables in the first set (function assumes that all of "data" will be analysed)
### (3) "correlation" is a T/F option that asks whether you want to do the analysis on the correlation (T) or covariance (F) matrix
############	OUTPUT	###############
### (1) a correlation (or covariance, depending on the option you choose) matrix of the original variables
### (2) the maximum possible covariance between all variables
### (3) the proportion of the maximum possible covariance that is described by the analysis (this is often quite low, 
### 	even with strong relationships among variables)
### (4) the eigenvalues (as "d")
### (5) the eigenvectors that describe how the original variables load onto the 2B-PLS variables 
###		("u" for variable set 1, and "v" for variable set 2)
### (6) the correlation between the 2B-PLS variables in set one and two, give for each axis

tbpls<-function(data,nvar1,correlation=T)
{
	if (correlation){
		cov.data<-cor(data)
	} else {
		cov.data<-cov(data)
	}
	max.covar<-sum(sqrt(diag(cov.data[1:nvar1,1:nvar1])))*sum(sqrt(diag(cov.data[(nvar1+1):ncol(data),(nvar1+1):ncol(data)]))) #maximum possible covariance among sets of variables
	tbpls.data<-svd(cov.data[1:nvar1,(nvar1+1):ncol(data)]) 
	prop.covar<-sum(tbpls.data$d)/max.covar #gives amount of potential covariance that is described by 2B-PLS
	tbpls.data$u<-tbpls.data$u*(-1) #for some reason R turns out negative values (or at least opposite to Rohlf and Corti)
	tbpls.data$v<-tbpls.data$v*(-1) #for some reason R turns out negative values (or at least opposite to Rohlf and Corti)
	rownames(tbpls.data$u)<-colnames(data)[1:nvar1] 
	rownames(tbpls.data$v)<-colnames(data)[(nvar1+1):ncol(data)] 
	ndim<-min(c(nvar1,(ncol(data)-nvar1))) #number of 2B-PLS dimensions
	scores<-matrix(ncol=(2*ndim),nrow=nrow(data)) 
	rownames(scores)<-rownames(data) 
	colnames(scores)[1:ndim]<-colnames(scores)[(ndim+1):ncol(scores)]<-1:ndim 
	for (j in 1:ndim){
		for (k in 1:nrow(data)){
			scores[k,j]<-sum(t(tbpls.data$u[,j])*data[k,1:nvar1]) #new morphology variable
			scores[k,(j+ndim)]<-sum(t(tbpls.data$v[,j])*data[k,(nvar1+1):ncol(data)]) #new performance variable
		}
	}
	correlation<-numeric(ndim) #calculate the correlation for each axis of the 2B-PLS latent variables
	for (j in 1:ndim){
		correlation[j]<-cor(scores[,j],scores[,(j+ndim)]) 
	}
	return(list(cov.matrix=cov.data,maximum.covariance=max.covar,prop.covar.explained=prop.covar,two.block=tbpls.data,axes.correlation=correlation)) 
}


### This is a test of significance of individual 2B-PLS axes, as in Rohlf and Corti 2000
########## 	INPUT	############
### All input is the same as the above function, with the addition of the number of 
### "nperms," which is the number of permutations you choose to do
### One change is "sumtest," which allows one to choose one of two tests: the default ("sumtest=FALSE") compares
### the axes independently - in other words, it does not consider how many variation has already been explained
### in lower PC dimensions.  This seems reasonable but can give strange results, as in PC2 not being significant
### but higher axes that are significant.  The "sumtest=TRUE" option sums lower dimensions and asks whether each 
### PC dimension and all lower dimensions together explain more variation than random.  This can be overly liberal
### because PC1 often explains so much variation that when lumped with higher dimensions everything becomes significant
########## 	OUTPUT	############
### (1) Singular values (on which the test of significance is based); this should be the same as obtained above
### (2) P-values, given for each axis
choose.axes.2BPLS<-function(data,nvar1,correlation=T,sumtest=FALSE,nperms=1000)
{
	nvar2<-ncol(data)-nvar1
	if (correlation){
		data<-scale(data)
		}
	ntax<-nrow(data) 
	data.1<-as.data.frame(data[,1:nvar1]) 
	data.2<-as.data.frame(data[,(nvar1+1):(nvar1+nvar2)]) 
	sing.values<-matrix(ncol=min(c(nvar1,nvar2)),nrow=nperms) 
	sing.values[1,]<-svd(cov(data)[1:nvar1,(1+nvar1):(nvar1+nvar2)])$d
	for (j in 1:(nperms-1)){
		rand.list<-sample(1:ntax,ntax) 
		permute.data<-as.matrix(cbind(data.1,data.2[rand.list,])) 
		big.R<-cov(permute.data) 
		R12<-big.R[1:nvar1,(1+nvar1):(nvar1+nvar2)] 
		sing.values[j+1,]<-svd(R12)$d 
	}
	P<-vector(len=ncol(sing.values))
	if (!sumtest){ ### Old test that compares each axis individually
		for (i in 1:ncol(sing.values)){
			P[i]<-(length(sing.values[sing.values[,i]>sing.values[1,i],i])+1)/nperms 
		}
	} else { ### New test that sums lower dimensions for test (avoids strange results but seems VERY liberal)
		mod.sing<-matrix(ncol=min(c(nvar1,nvar2)),nrow=nperms) 
		### First get sums of eigenvalues
		for (i in 1:nperms){ 
			for (j in 1:ncol(sing.values)){
				mod.sing[i,j]<-sum(sing.values[i,seq(j)])
			}
		}
		### Now get P-values
		for (i in 1:ncol(mod.sing)){
			P[i]<-(length(mod.sing[mod.sing[,i]>mod.sing[1,i],i])+1)/nperms 
		}
	}	
	list(sample.sing.values=sing.values[1,],P.values=P) 
}














