#comparing distances among ECD-style taxa (in morphology and performance PC space)
####  	INPUT	####
# (1) Data frame with:
#	(1a) a "location" vector (in this case, China, Colombia, and Australia) as the first column
#	(1b) an "ecology" vector (in this case, microhabitat use) as the second column
#	(1c) all remaining columns continuous variables (e.g., morphology or performance) that
#		are assumed independent, for example as might be obtained from a principal components analysis
# (2) "species1" is a list of the species for comparison from the first location 
# (3) "species2" is a list of the species for comparison from the second location
# NOTE: the ecology and locations of these species will be extracted from the data frame
####  	OUTPUT	####
# (1) "Focal.MPD" is the mean pairwise difference among focal taxa across the two locations
# (2) "Loc.MPD" is the mean pairwise difference among all pairs of species across the two
#		locations (but no distances are from within-location pairs)
# (3) "num.Loc" is the number of pairwise comparisons for location
# (4) "P.Loc" is the proportion of the pairwise comparisons for location that are smaller than the focal value
# (5) "Ecol.MPD" is the mean pairwise difference among all pairs of species with the same ecology as the focal species
# (6) "num.Ecol" is the number of pairwise comparisons of species with the same ecology as the focal species
# (7) "P.Ecol" is the proportion of the pairwise comparisons for ecology that are smaller than the focal value

ecd.distance<-function(data,species1,species2){
	locations<-levels(data[,1])[unique(c(data[species1,1],data[species2,1]))] # Funky indexing because "unique" return a numeric
	ecology<-levels(data[,2])[unique(c(data[species1,2],data[species2,2]))] # Funky indexing because "unique" return a numeric
	if (length(ecology)>1) stop("Your species do not have the same ecology in the data matrix")	
	if (length(locations)>2) stop("Your species are from more than two geographic areas")
	if (length(locations)==1) stop("Your species are from the same location")
	dist.matrix<-as.matrix(dist(data[,3:ncol(data)]))#produces n by n matrix of all pairwise Euclidean distances just lower triangular, which makes subsequent computations much easier
	ncom<-length(dist.matrix)#number of pairwise comparisons
	#sum(dist.sample)#sum of all pairwise distances
	#hist(dist.sample)#histogram of all pairwise distances
	### First calculate the distance among focal species
	foc.dist<-mean(dist.matrix[species1,species2])
	### Next calculate the mean distance among pairs of species in the two locations
	loc1<-rownames(data)[data[,1]==locations[1]]
	loc2<-rownames(data)[data[,1]==locations[2]]
	dist.loc<-dist.matrix[loc1,loc2]
	num.loc<-length(dist.loc) # Number of comparisons
	mean.loc<-mean(dist.loc) # Mean pairwise distance
	P.loc<-sum(dist.loc<=foc.dist)/num.loc
	### Next calculate the mean distance among species with similar ecology		
	dist.ecol<-dist(data[data[,2]==ecology,3:ncol(data)])
	mean.ecol<-mean(dist.ecol) # Mean pairwise distance
	num.ecol<-length(dist.ecol) # Number of comparisons
	P.ecol<-sum(dist.ecol<=foc.dist)/num.ecol
	output<-c(foc.dist,mean.loc,num.loc,P.loc,mean.ecol,num.ecol,P.ecol)
	names(output)<-c("Focal.MPD","Loc.MPD","num.Loc","P.Loc","Ecol.MPD","num.Ecol","P.Ecol")
	return(list(results=output,ecology=ecology,locations=locations))
}	
