####################################################################################################################
#########	(1) Test for effect of history: calculate distance between new optimum and ancestral optimum, 	########
#########	as well as between ancestral optimum and current value of group within the new regime,			########
#########	then see if latter distance is significantly less than former (as an effect of history might 	########
#########	be suggested by inability to move from from ancestral state/optimum								########
####################################################################################################################
####################################################################################################################
#########	(2) Test for convergence: calculate distance between new optimum and new taxa in that optimum, 	########
#########	as well as between ancestral optimum and current value of group within the new regime,			########
#########	then see if latter distance is significantly greater than former as expected by convergence 	########
####################################################################################################################
####################################################################################################################
#########	(3) Test for convergence: previous distance test does not consider direction of divergence; 	########
#########	we would expect it to be largely toward the new optimum in multivariate space.  This test 		########
#########	allows us to test the correlation between the expected and observed divergence vectors, and  	########
#########	we further combine this test with the above distance test to ask how simulation runs produce 	########
#########	divergence vectors BOTH closer to the new optimum and toward the new optimum than random		########
####################################################################################################################

##########	 INPUT	 #############
### (1) "tree" is the phylogeny of all species in the the analysis
### (2) "data" is a data frame that has all species, with 1st column a vector of selective environments, 2nd column designating species as part of the focal group 
###  	or not ("Y" or "N", respectively), and all remaining columns the phenotypic (continuous) data. Row names should correspond to the tip names of the phylogeny.
### (3) "regimes" is a vector with first element the name of the ancestral selective environment, and the remaining elements the new selective regimes
### (4) "OU.opt" allows one to use optima that come from an OU analysis of the entire phylogeny. This should be a data frame with row names corresponding to 
###		the new selective environment(s), as in "regimes"
###	(5) "nsim" is the number of simulations that are done for tests of significance
### (6) "hist.test" allows you to test the effect of history
### (7) "conv.test" allows you to do the two convergence tests
##########	 OUTPUT	 #############
### NOTE: for full explanation of these results, see Moen et al. 2013 Proc. R. Soc. and especially the supplementary methods
### (1) "results$results$history.test" gives expected and observed divergence and the latter's P-value
### (2) "results$results$convergence.distance" gives distances among species in the same selective environment (regime) and between closely related species, 
###		as well as the associated P-value
### (3) "results$results$converge.vec.cor" gives the correlation between observed and expected divergence vectors, the angle between them, and the associated P-value
### (4) "anc.opt" gives the mean phenotype of species of the focal clade in the ancestral selective environment
###	(5) "focal.values" gives the mean phenotype of species of the focal clade in the new selective environment
### (6) "new.opt" gives the mean phenotype (or OU optimum, if chosen) of the non-focal species in the new selective environment
### (7) "global.P" is a test of whether all results together (across multiple new selective environments) are consistent with history or convergence
### (8) "converge.P" is a test of whether the distance and vector correlation together are more consistent with convergence than expected by chance (i.e., instead
###		of evaluating them independently)

library(ape) # Needed for geiger
library(geiger) # Function uses some commands from Geiger for simulation

converge.test.complete<-function(tree,data,regimes,OU.opt=NULL,nsim=1000,hist.test=T,conv.test=T) # Function will do everything, but options remain to leave out tests
{
	if (!hist.test & !conv.test) {stop("You need at least one test!")}
	n<-length(regimes)-1 # This defines how many comparisons will be made
	nvars<-(ncol(data)-2) 
	if (!is.null(OU.opt)) OU.opt<-OU.opt[regimes[2:(n+1)],] #Force order to make things easier downstream
	
	### First calculate the "starting point", defined as the mean of the focal group species in the ancestral regime
	anc.foc<-rownames(data)[data[,1]==regimes[1] & data[,2]=="Y"]  # Defining the taxa that are that are (1) part of the focal group but (2) in the ancestral regime
	anc.foc.dat<-data[anc.foc,3:ncol(data)] # Phenotypic data from only the new.foc group in the ancestral regime
	mean.anc.foc<-apply(anc.foc.dat,MARGIN=2,FUN=mean) 
	
	### Next calculate the evolutionary optima
	mean.new.opt<-matrix(nrow=n,ncol=nvars)
	rownames(mean.new.opt)<-regimes[2:(n+1)]
	colnames(mean.new.opt)<-names(mean.anc.foc) 
	if (is.null(OU.opt)){
		new.opt.list<-list() # Taxon lists for simulation	
		for (i in 1:n){
			new.opt.list[[i]]<-rownames(data)[data[,1]==regimes[1+i] & data[,2]=="N"] # These conditions correspond to (1) new regime and (2) not focal group
			mean.new.opt[i,]<-apply(data[new.opt.list[[i]],3:ncol(data)],MARGIN=2,FUN=mean) # Defines optimum position as the mean of current taxa
		}
	} else {
		mean.new.opt<-OU.opt[rownames(mean.new.opt),] # Optimum defined by OU optimum found by phylogenetic analyses of current taxa
	}
	
	### Finally calculate the values of taxa in new regime
	mean.new.foc<-matrix(nrow=n,ncol=nvars)
	rownames(mean.new.foc)<-regimes[2:(n+1)]
	colnames(mean.new.foc)<-names(mean.anc.foc)
	new.foc.list<-list() # Taxon list for simulation
	for (i in 1:n){
		new.foc.list[[i]]<-rownames(data)[data[,1]==regimes[1+i] & data[,2]=="Y"] # Defining the taxa that are (1) the focus in (2) the new regime
		new.foc.dat<-data[new.foc.list[[i]],3:ncol(data)] # Phenotypic data from only the new.foc group in the new regime
		if (is.vector(new.foc.dat)) {
			mean.new.foc[i,]<-new.foc.dat #this conditional allows the function to proceed even if there is just one species as the focus
		} else {
			mean.new.foc[i,]<-apply(new.foc.dat,MARGIN=2,FUN=mean)
		} 
	}	
	### Set up output matrices
	empty.mat<-matrix(ncol=3,nrow=n) #used a lot
	if (hist.test & conv.test){
		results.matrix<-list()
		col.nam.mat<-matrix(data=c("Expected","Observed","hist.P.value","same.regime","closely.related","conv.dist.P","vec.cor","angle","cor.P.value"),nrow=3,ncol=3,byrow=T)
		for (i in 1:3){
			results.matrix[[i]]<-empty.mat
			colnames(results.matrix[[i]])<-col.nam.mat[i,]
			rownames(results.matrix[[i]])<-regimes[2:(n+1)] 
		}
		names(results.matrix)<-c("history.test","converge.distance","converge.vec.cor")
		#colnames(results.matrix)<-c("Expected","Observed","hist.P.value","same.regime","closely.related","conv.dist.P","vec.cor","angle","cor.P.value")
		null.dist<-list(matrix(ncol=n,nrow=nsim),matrix(ncol=n,nrow=nsim),matrix(ncol=n,nrow=nsim)) 
	} else {
		if (hist.test){
			results.matrix<-empty.mat
			colnames(results.matrix)<-c("Expected","Observed","ind.P.value") 
			rownames(results.matrix)<-regimes[2:(n+1)] 
			null.dist<-matrix(ncol=n,nrow=nsim) # n columns for the single test
		} else {
			results.matrix<-list()
			col.nam.mat<-matrix(data=c("same.regime","closely.related","conv.dist.P","vec.cor","angle","cor.P.value"),nrow=2,ncol=3,byrow=T)
			for (i in 1:2){
				results.matrix[[i]]<-empty.mat
				colnames(results.matrix[[i]])<-col.nam.mat[i,]
				rownames(results.matrix[[i]])<-regimes[2:(n+1)] 
			}	
			names(results.matrix)<-c("converge.distance","converge.vec.cor")
			null.dist<-list(matrix(ncol=n,nrow=nsim),matrix(ncol=n,nrow=nsim)) 
		}
	}	

	### Calculate empirical results
	if (hist.test & conv.test){
			for (i in 1:n){
				results.matrix[[1]][i,1]<-sqrt(sum((mean.new.opt[i,]-mean.anc.foc)^2)) #Expected distance for adaptive evolution
				results.matrix[[1]][i,2]<-sqrt(sum((mean.new.foc[i,]-mean.anc.foc)^2)) #Observed divergence from ancestral regime
				null.dist[[1]][1,i]<-results.matrix[[1]][i,1]/results.matrix[[1]][i,2] # History statistic; first divided by second, as I expect the first to be larger if history is important
				results.matrix[[2]][i,1]<-sqrt(sum((mean.new.opt[i,]-mean.new.foc[i,])^2)) #Distance between two unrelated groups in the same selective regime
				results.matrix[[2]][i,2]<-sqrt(sum((mean.new.foc[i,]-mean.anc.foc)^2)) #Observed divergence from ancestral regime between closely related groups
				null.dist[[2]][1,i]<-results.matrix[[2]][i,1]/results.matrix[[2]][i,2] # Convergence by distance statistic; first divided by second, as I expect the first to be smaller
				dummy.project<-vec.cor(begin.vec=mean.anc.foc,exp.end=mean.new.opt[i,],obs.end=mean.new.foc[i,])
				results.matrix[[3]][i,1]<-dummy.project$correlation #correlation
				results.matrix[[3]][i,2]<-dummy.project$angle #angle in degrees
				null.dist[[3]][1,i]<-results.matrix[[3]][i,1] # Convergence by correlation statistic
			}
	} else {
		if (hist.test){
			results.matrix[,1]<-apply(mean.new.opt,MARGIN=1,function(x){sqrt(sum((x-mean.anc.foc)^2))})
			results.matrix[,2]<-apply(mean.new.foc,MARGIN=1,function(x){sqrt(sum((x-mean.anc.foc)^2))})			
			null.dist[1,]<-apply(results.matrix,MARGIN=1,function(x){x[1]/x[2]})
		} else {
			for (i in 1:n){
				results.matrix[[1]][i,1]<-sqrt(sum((mean.new.opt[i,]-mean.new.foc[i,])^2)) #Distance between two unrelated groups in the same selective regime
				results.matrix[[1]][i,2]<-sqrt(sum((mean.new.foc[i,]-mean.anc.foc)^2)) #Observed divergence from ancestral regime between closely related groups
				null.dist[[1]][1,i]<-results.matrix[[1]][i,1]/results.matrix[[1]][i,2] # Convergence by distance statistic; first divided by second, as I expect the first to be smaller
				dummy.project<-vec.cor(begin.vec=mean.anc.foc,exp.end=mean.new.opt[i,],obs.end=mean.new.foc[i,])
				results.matrix[[2]][i,1]<-dummy.project$correlation #correlation
				results.matrix[[2]][i,2]<-dummy.project$angle #angle in degrees
				null.dist[[2]][1,i]<-results.matrix[[2]][i,1] # Convergence by correlation statistic
			}	
		}
	}	
	
	### Now simulation for statistics
	phyl.var<-geiger:::vcv.phylo(tree,data[tree$tip.label,names(mean.anc.foc)]) 
	phyl.var<-diag(diag(phyl.var)) #remove covariances, as they are 0 and any other value is estimation error.  Forcing them to 0 here should result in faster computation
	sim.rand<-sim.char(phy=tree, par=phyl.var, nsim = (nsim-1), model = "BM" , root = 0) 
	
	### Calculate null distributions (conditionals are complicated, but it prevents evaluating them nsim times, instead just once depending on the type of test
	if (is.null(OU.opt)){
		if (hist.test & conv.test){
			for (j in 1:(nsim-1)){
				sim.new.opt<-matrix(nrow=n,ncol=nvars) 
				sim.new.foc<-matrix(nrow=n,ncol=nvars) 
				sim.anc.foc<-apply(sim.rand[,,j][anc.foc,],MARGIN=2,FUN=mean) 
				for (k in 1:n){
					sim.new.opt[k,]<-apply(sim.rand[,,j][new.opt.list[[k]],],MARGIN=2,FUN=mean) 
					if (length(new.foc.list[[k]])<=1) {
						sim.new.foc[k,]<-sim.rand[,,j][new.foc.list[[k]],]#force code to consider the original matrix, which will have the sim.new.opt dimensionality
						} else {
						sim.new.foc[k,]<-apply(sim.rand[,,j][new.foc.list[[k]],],MARGIN=2,FUN=mean)
						} 
					### Here add new tests
					dis<-numeric(len=3)
					dis[1]<-sqrt(sum((sim.new.foc[k,]-sim.anc.foc)^2)) # Observed trajectory / closely related
					dis[2]<-sqrt(sum((sim.new.opt[k,]-sim.anc.foc)^2)) # Expected trajectory of evolution
					dis[3]<-sqrt(sum((sim.new.opt[k,]-sim.new.foc[k,])^2)) # same regime
					null.dist[[1]][j+1,k]<-dis[2]/dis[1] # History test 
					null.dist[[2]][j+1,k]<-dis[3]/dis[1] # Convergence distance test
					null.dist[[3]][j+1,k]<-vec.cor(begin.vec=sim.anc.foc,exp.end=sim.new.opt[k,],obs.end=sim.new.foc[k,])$correlation # Vector correlation test
				}
			}
		} else {
			### Fix this to reflect current output matrices
			if (hist.test){
				for (j in 1:(nsim-1)){
					sim.new.opt<-matrix(nrow=n,ncol=nvars) 
					sim.new.foc<-matrix(nrow=n,ncol=nvars) 
					sim.anc.foc<-apply(sim.rand[,,j][anc.foc,],MARGIN=2,FUN=mean) 
					for (k in 1:n){
						sim.new.opt[k,]<-apply(sim.rand[,,j][new.opt.list[[k]],],MARGIN=2,FUN=mean) 
						if (length(new.foc.list[[k]])<=1) {
							sim.new.foc[k,]<-sim.rand[,,j][new.foc.list[[k]],]#force code to consider the original matrix, which will have the sim.new.opt dimensionality
						} else {
							sim.new.foc[k,]<-apply(sim.rand[,,j][new.foc.list[[k]],],MARGIN=2,FUN=mean)
						} 
						dist1<-sqrt(sum((sim.new.opt[k,]-sim.anc.foc)^2))
						dist2<-sqrt(sum((sim.new.foc[k,]-sim.anc.foc)^2))
						null.dist[j+1,k]<-dist1/dist2 
					}
				}
			} else {
			# Only convergence tests here
				for (j in 1:(nsim-1)){
					sim.new.opt<-matrix(nrow=n,ncol=nvars) 
					sim.new.foc<-matrix(nrow=n,ncol=nvars) 
					sim.anc.foc<-apply(sim.rand[,,j][anc.foc,],MARGIN=2,FUN=mean) 
					for (k in 1:n){
						sim.new.opt[k,]<-apply(sim.rand[,,j][new.opt.list[[k]],],MARGIN=2,FUN=mean) 
						if (length(new.foc.list[[k]])<=1) {
							sim.new.foc[k,]<-sim.rand[,,j][new.foc.list[[k]],]#force code to consider the original matrix, which will have the sim.new.opt dimensionality
						} else {
							sim.new.foc[k,]<-apply(sim.rand[,,j][new.foc.list[[k]],],MARGIN=2,FUN=mean)
						}	 
						null.dist[[1]][j+1,k]<-sqrt(sum((sim.new.opt[k,]-sim.new.foc[k,])^2))/sqrt(sum((sim.new.foc[k,]-sim.anc.foc)^2)) #same regime divided by closely related pairs
						null.dist[[2]][j+1,k]<-vec.cor(begin.vec=sim.anc.foc,exp.end=sim.new.opt[k,],obs.end=sim.new.foc[k,])$correlation # Vector correlation test
					}
				}
			}	
		}	
	} else {
### Here is where OU optima come in (should be cut-and-pasted from above, minus code about OU.optima)
		sim.new.opt <- OU.opt # This forces the OU optima
		if (hist.test & conv.test){
			for (j in 1:(nsim-1)){
				sim.new.foc<-matrix(nrow=n,ncol=nvars) 
				sim.anc.foc<-apply(sim.rand[,,j][anc.foc,],MARGIN=2,FUN=mean) 
				for (k in 1:n){
					if (length(new.foc.list[[k]])<=1) {
						sim.new.foc[k,]<-sim.rand[,,j][new.foc.list[[k]],]#force code to consider the original matrix, which will have the sim.new.opt dimensionality
						} else {
						sim.new.foc[k,]<-apply(sim.rand[,,j][new.foc.list[[k]],],MARGIN=2,FUN=mean)
						} 
					dis<-numeric(len=3)
					dis[1]<-sqrt(sum((sim.new.foc[k,]-sim.anc.foc)^2)) # Observed trajectory / closely related
					dis[2]<-sqrt(sum((sim.new.opt[k,]-sim.anc.foc)^2)) # Expected trajectory of evolution
					dis[3]<-sqrt(sum((sim.new.opt[k,]-sim.new.foc[k,])^2)) # same regime
					null.dist[[1]][j+1,k]<-dis[2]/dis[1] # History test 
					null.dist[[2]][j+1,k]<-dis[3]/dis[1] # Convergence distance test
					null.dist[[3]][j+1,k]<-vec.cor(begin.vec=sim.anc.foc,exp.end=sim.new.opt[k,],obs.end=sim.new.foc[k,])$correlation # Vector correlation test
				}
			}
		} else {
			if (hist.test){
				for (j in 1:(nsim-1)){
					sim.new.foc<-matrix(nrow=n,ncol=nvars) 
					sim.anc.foc<-apply(sim.rand[,,j][anc.foc,],MARGIN=2,FUN=mean) 
					for (k in 1:n){
						if (length(new.foc.list[[k]])<=1) {
							sim.new.foc[k,]<-sim.rand[,,j][new.foc.list[[k]],]#force code to consider the original matrix, which will have the sim.new.opt dimensionality
						} else {
							sim.new.foc[k,]<-apply(sim.rand[,,j][new.foc.list[[k]],],MARGIN=2,FUN=mean)
						} 
						dist1<-sqrt(sum((sim.new.opt[k,]-sim.anc.foc)^2))
						dist2<-sqrt(sum((sim.new.foc[k,]-sim.anc.foc)^2))
						null.dist[j+1,k]<-dist1/dist2 
					}
				}
			} else {
				for (j in 1:(nsim-1)){
					sim.new.foc<-matrix(nrow=n,ncol=nvars) 
					sim.anc.foc<-apply(sim.rand[,,j][anc.foc,],MARGIN=2,FUN=mean) 
					for (k in 1:n){
						if (length(new.foc.list[[k]])<=1) {
							sim.new.foc[k,]<-sim.rand[,,j][new.foc.list[[k]],]#force code to consider the original matrix, which will have the sim.new.opt dimensionality
						} else {
							sim.new.foc[k,]<-apply(sim.rand[,,j][new.foc.list[[k]],],MARGIN=2,FUN=mean)
						} 
						null.dist[[1]][j+1,k]<-sqrt(sum((sim.new.opt[k,]-sim.new.foc[k,])^2))/sqrt(sum((sim.new.foc[k,]-sim.anc.foc)^2)) #same regime divided by closely related pairs
						null.dist[[2]][j+1,k]<-vec.cor(begin.vec=sim.anc.foc,exp.end=sim.new.opt[k,],obs.end=sim.new.foc[k,])$correlation # Vector correlation test
					}
				}
			}	
		}
	}	
	### Individual and global P-values
	if (hist.test & conv.test){
		#First individual P-values
		results.matrix[[1]][,3]<-apply(null.dist[[1]],MARGIN=2,function(x){sum(x>=x[1])/length(x)}) # History test
		results.matrix[[2]][,3]<-apply(null.dist[[2]],MARGIN=2,function(x){sum(x<=x[1])/length(x)}) # Convergence distance test
		results.matrix[[3]][,3]<-apply(null.dist[[3]],MARGIN=2,function(x){sum(abs(1-x)<=abs(1-x[1]))/length(x)})  # Convergence correlation test
		# Global P-value
		global.P<-vector(len=3) #P.value output
		names(global.P)<-c("History","Conv.dist","Conv.vec.cor")
		global.P[1]<-sum(n==apply(null.dist[[1]],MARGIN=1,function(x){
			sum(x>=null.dist[[1]][1,])}))/nsim # First, global P-value for history test
		global.P[2]<-sum(n==apply(null.dist[[2]],MARGIN=1,function(x){
			sum(x<=null.dist[[2]][1,])}))/nsim # Second, convergence distance test
		global.P[3]<-sum(n==apply(null.dist[[3]],MARGIN=1,function(x){
			sum(abs(1-x)<=abs(1-null.dist[[3]][1,]))}))/nsim # Finally, vector correlation test for convergence
		#Combined convergence P-value for passing both distance and direction random tests (significant if passed both on same simulation)
		converge.P<-vector(len=n)
		names(converge.P)<-regimes[2:(n+1)]
		ty<-apply(null.dist[[2]],MARGIN=2,function(x){x<=x[1]}) # Convergence distance test
		tz<-apply(null.dist[[3]],MARGIN=2,function(x){abs(1-x)<=abs(1-x[1])})  # Convergence correlation test
		converge.P<-apply(ty & tz,MARGIN=2,sum)/nsim
		names(converge.P)<-regimes[2:(n+1)]
		return(list(results=results.matrix,anc.opt=round(mean.anc.foc,digits=4),focal.values=round(mean.new.foc,digits=4),
			new.opt=round(mean.new.opt,digits=4),global.P=global.P, converge.P=converge.P))		
	} else {
		if (hist.test){
			results.matrix[,3]<-apply(null.dist,MARGIN=2,function(x){sum(x>=x[1])/length(x)})
			global.P<-sum(n==apply(null.dist,MARGIN=1,function(x){
				sum(x>=null.dist[1,])}))/nsim # First, global P-value for history test
			return(list(results=results.matrix,anc.opt=round(mean.anc.foc,digits=4),focal.values=round(mean.new.foc,digits=4),
				new.opt=round(mean.new.opt,digits=4),global.P=global.P))		
		} else {
			results.matrix[[1]][,3]<-apply(null.dist[[1]],MARGIN=2,function(x){sum(x<=x[1])/length(x)})  # Convergence distance test
			results.matrix[[2]][,3]<-apply(null.dist[[2]],MARGIN=2,function(x){sum(abs(1-x)<=abs(1-x[1]))/length(x)})  # Convergence correlation test
			global.P<-vector(len=2) #P.value output
			names(global.P)<-c("Conv.dist","Conv.vec.cor")
			global.P[1]<-sum(n==apply(null.dist[[1]],MARGIN=1,function(x){
				sum(x<=null.dist[[1]][1,])}))/nsim # Second, convergence distance test
			global.P[2]<-sum(n==apply(null.dist[[2]],MARGIN=1,function(x){
				sum(abs(1-x)<=abs(1-null.dist[[2]][1,]))}))/nsim # Finally, vector correlation test for convergence
			#Combined convergence P-value for passing both distance and direction random tests (significant if passed both on same simulation)
			converge.P<-vector(len=n)
			ty<-apply(null.dist[[1]],MARGIN=2,function(x){x<=x[1]}) # Convergence distance test
			tz<-apply(null.dist[[2]],MARGIN=2,function(x){abs(1-x)<=abs(1-x[1])})  # Convergence correlation test
			converge.P<-apply(ty & tz,MARGIN=2,sum)/nsim
			names(converge.P)<-regimes[2:(n+1)]
			return(list(results=results.matrix,anc.opt=round(mean.anc.foc,digits=4),focal.values=round(mean.new.foc,digits=4),
				new.opt=round(mean.new.opt,digits=4),global.P=global.P, converge.P=converge.P))		
		}
	}		
}
### Additional necessary function for the above command
### Standardized projection of observed change to expected (AKA vector correlation)
### INPUT and OUTPUT are given for reference purposes, if you want to use this in other applications
##########	 INPUT	 #############
### (1) "exp.end" is the end of one vector, here the phenotype of putatively convergent taxa (the expected result of convergence)
### (2) "obs.end" is the end of a second vector, here the phenotype of the focal group in the novel selective environment (which partly represents
###		the observed divergence from the ancestral phenotype)
###	(3) "begin.vec" is the ancestral phenotype, which defines the start of both the observed and expected divergence vectors.  If this is the origin (which 
###		might be the case in a general vector correlation), then the default "NULL" can be used
###	Note: Both starting points and end points of vector can be used by substitution (and an extra input) in the function, but here there is a common starting point
##########	 OUTPUT	 #############
### (1) "correlation" is the vector correlation
### (2) "angle" is the angle between the two vectors

vec.cor<-function(exp.end,obs.end,begin.vec=NULL)
{
	if (is.null(begin.vec)){
		begin.vec<-rep(0,length(exp.end))
	} # Default is to start at origin
	begin.vec<-as.numeric(matrix(begin.vec,byrow=T))#force all of these into column vectors
	exp.end<-as.numeric(matrix(exp.end,byrow=T))
	obs.end<-as.numeric(matrix(obs.end,byrow=T))	
	dif1<-(exp.end-begin.vec)#change axis of expectation
	dif2<-(obs.end-begin.vec)#change to focal spot from ancestor
	d1<-sqrt(sum(dif1^2))#length of change vector
	d2<-sqrt(sum(dif2^2))#length of change vector
	correlation<-t(dif1/d1)%*%(dif2/d2)#vector correlation
	angle<-acos(correlation)/pi*180
	return(list(correlation=correlation,angle=angle))
}